function [filtersvm,LOG]=lfilterregression(x,y,options)
% function thta permits to learn an optimal filter applied to x before
% classification
% Parameters:
% x : data
% y : classes( -1,1)
% options:
%  options.f : legth of the filter
%  options.F : initialisation of the filter
%  options.lambda : lambda regularization term
%  options.verbose: print everything
%  options.log : log informations
%  options.nbitermax : log informations
%  options.stopcF : threshold for stopping on F changes
%  options.decy : y time shift...
%  options.solver : solver choice for non-linear svm
%
% Result:
% sigsvm:
%  sigsvm.kernel : kernel type in SVM-KM format
%                   'gaussian' or 'linear' leading to primal or dual
%                    optimization
%  sigsvm.xsup : support vector
%  sigsvm.w    : dual or primal variables weight
%  sigsvm.w0   : dual or primal zero value
%  sigsvm.pos  : position of support vectors (if non linear)
%  sigsvm.F    : optimal filter
%  sigsvm.decy : time shift
%
%  Author : Remi Flamary remi (dot) flamary (at) gmail (dot) com


if ~isfield(options,'f'); options.f=10; end
if ~isfield(options,'r'); options.r=0; end
if ~isfield(options,'F'); options.F=ones(options.f,size(x,2)); end

if ~isfield(options,'kernel'); options.kernel='linear'; end
if ~isfield(options,'kerneloption'); options.kerneloption=1; end
if ~isfield(options,'lambda'); options.lambda=1; end
if ~isfield(options,'C'); options.C=100; end
if ~isfield(options,'verbose'); options.verbose=0; end
if ~isfield(options,'log'); options.log=0; end
if ~isfield(options,'nbitermax'); options.nbitermax=50; end
if ~isfield(options,'decy'); options.decy=0; end
if ~isfield(options,'alphainit'); options.alphainit=[]; end
if ~isfield(options,'solver'); options.solver=0; end
if ~isfield(options,'numericalprecision'); options.numericalprecision=1e-15; end
if ~isfield(options,'use_cg'); options.use_cg=0; end
if ~isfield(options,'regnorm2'); options.regnorm2=1; end
if ~isfield(options,'regnorm12'); options.regnorm12=0; end
if ~isfield(options,'regnormdiff'); options.regnormdiff=0; end
if ~isfield(options,'method'); options.method=0; end

if ~isfield(options,'multiclass'); options.multiclass=0; end
if ~isfield(options,'nbclass'); options.nbclass=length(unique(y)); end

if ~isfield(options,'sel'); options.sel=[]; end
if ~isfield(options,'selsparse'); options.selsparse=0; end

if size(y,2)>1
    options.multiclass=1;
end

t0=cputime;



if options.multiclass==0
    % initialization
    F=options.F;
    f=options.f;
    
    LOG=[];
    
  %  Yt=DecalageSignal(y,options.decy);
    %Xt=mfilter(F,X);


    % counter initialisation
    global nbcall
    nbcall=0;

    % data initialisation
    

    data=options;
    data.X=x;
    data.Y=y;
    data.options=options;



    % solver options initialisation
    
    switch options.method
        
        case 1
            
            [vars,LOG]=solve_filter_ridge(data,options);
            
        case 2
            
            [vars,LOG]=solve_filter_group(data,options);
        
        otherwise
            
            
            [vars,LOG]=solve_real_filter(data,options);
        
    end
            
   

    % returned structure
    filtersvm.kernel='linear';
    filtersvm.kerneloption=options.kerneloption;
    filtersvm.xsup =vars.xsup;
    filtersvm.w   =vars.w;
    filtersvm.w0  =vars.w0;
    filtersvm.pos =vars.pos;
    filtersvm.F   =vars.F;
    filtersvm.decy=options.decy;
    filtersvm.nbclass=options.nbclass;
    filtersvm.method=options.method;


else % multiclass

    vals=sort(unique(y),'ascend');

    if options.verbose~=0
        disp(['Multiclass regression (' num2str(length(vals)) 'classes)' ]);
    end

    switch options.multiclass

        case 1 % one against all


            options2=options;
            options2.multiclass=0;
            options2.nbclass=size(y,2);
            
            for i=1:size(y,2)

                if options.verbose~=0
                    disp(['Regression :' num2str(i) '/' num2str(length(vals)) ]);
                end;
                Yt=y(:,i);
                [models{i},LOG{i}]=lfilterregression(x,Yt,options2);

            end

            filtersvm.multiclass=options.multiclass;
            filtersvm.nbclass=size(y,2);
            filtersvm.models=models;
            filtersvm.kerneloption=options.kerneloption;
            filtersvm.kernel=options.kernel;

    end




end

end


function [vars,LOG]=solve_filter_ridge(data,options)

LOG=[];

d=size(data.X,2);
f=options.f;

vars.w=ones(d,1);
vars.xsup=[];
vars.pos=[];


X=[time_delay_embedding(data.X,options) ones(size(data.Y,1),1)];

noregeye=eye(size(X,2)); noregeye(end)=0;

% ridrge regression on tde
F=(X'*X + options.regnorm2*noregeye)\X'*data.Y;

vars.w0=F(end);
vars.F=(reshape(F(1:end-1),f,d));

end

function [vars,LOG]=solve_filter_group(data,options)
LOG=[];
d=size(data.X,2);
f=options.f;

vars.w=ones(d,1);
vars.xsup=[];
vars.pos=[];


X=[time_delay_embedding(data.X,options)];

groups=[];
for i=1:d
   groups=[groups i*ones(1,f)];
end

options.loss=0;
options.lambda=options.regnorm12;


options.groups=groups;

[svm]=lgrouplearn(X,data.Y,options);

vars.w0=svm.w0;
vars.F=(reshape(svm.w,f,d));

end

function [vars,LOG]=solve_real_filter(data,options)

   
    Yt=DecalageSignal(data.Y,options.decy);
    %Xt=mfilter(F,X);
    
   % counter initialisation
    global nbcall
    nbcall=0;

    data.Y=Yt;

    F=options.F;

    % solver options initialisation
            
    opts=options;
        
        
%     if isempty(options.sel)


        opts.costfunc=@costfunc_linear;
        opts.gradfunc=@gradfunc_linear;

    % solving
    if options.regnorm12==0
        % solving standard gradient descent problem problem
        [F,vars,costmin,LOG2]=grad_desc(F,[],data,opts);
    else
        % solving reweighted
        if ~isempty(options.sel)
            data.Y=Yt;
        end
        opts2=opts;
        opts2.updatefunc=@reweightedupdate;
        [F,vars,costmin,LOG2]=iterative_solver(F,[],data,opts2);

    end


    if options.log==1

        LOG.Ftot=LOG2.xtot;
        LOG.F=options.F;
        LOG.decy=options.decy;
        LOG.J=LOG2.J;
        LOG.time=LOG2.time;
        LOG.xsup=LOG2.vars.xsup;
        LOG.pos=LOG2.vars.pos;
        LOG.w=LOG2.vars.w;
        LOG.w0=LOG2.vars.w0;
        LOG.kernel=options.kernel;
        LOG.kerneloption=options.kerneloption;
        LOG.nbcostcall=nbcall;
    end
    
    vars.F=F;


end

function [F,costnew,varsnew]=reweightedupdate(F_1,vars,data)
% update function for reweignted

if isempty(vars)
    % initialisation
    options=data.options;
    regnorm2=options.regnorm12;
    options.regnorm12=0;
    options.regnorm2=regnorm2*ones(size(F_1,2),1);

    if options.verbose~=0
        options.verbose=options.verbose+1;
    end
    vars.options=options;
    vars.regnorm=regnorm2;
    vars.options.decy=0;
    vars.LOG2=[];

end


vars.options.F=F_1;

% solve l2 problem
[filtersvm,LOG]=lfilterregression(data.X,data.Y,vars.options);

F=filtersvm.F;

costnew=costfunc_linear(F,data,filtersvm);


% reweighting!!
vars.options.F=F;
wgth=sqrt(sum(F.*F));
vars.options.regnorm2=  vars.regnorm./wgth;
vars.w=filtersvm.w;
vars.w0=filtersvm.w0;
vars.pos=filtersvm.pos;
vars.xsup=filtersvm.xsup;


varsnew=vars;

if vars.options.log~=0
    varsnew.LOG2{length(varsnew.LOG2)+1}=LOG;
end



end



function [costnew,varsnew]=costfunc_linear(F,data,vars)
% cost function for the non-linear case

%cost counter
global nbcall
nbcall=nbcall+1;

varsnew=vars;
pos=[];xsup=[];

% filtering data
Xf=[mfilter(F,data.X) ones(size(data.X,1),1)];


noregeye=eye(size(Xf,2)); noregeye(end)=0;
%opt.lin_cg=1;
%solving linear system
%[w,w0,obj]=lsvmclass(Xf,data.Y,data.C);
W=(Xf'*Xf+ data.lambda*noregeye)\Xf'*data.Y;
w=W(1:end-1);
w0=W(end);

% linear cost
costnew=data.lambda*1/2*sum(w.*w)+1/2*sum((data.Y-(Xf*W)).^2)+1/2*Omega2(F,data.regnorm2)+data.regnorm12*Omega12(F)+data.regnormdiff*Omegadiff(F);

%costnew=obj+1/2*Omega2(F,data.regnorm2)+data.regnorm12*Omega12(F);


varsnew.pos=pos;
varsnew.w=w;
varsnew.w0=w0;
varsnew.xsup=xsup;

end

function [gradf]=gradfunc_linear(F,data,vars0)
% gradient function for the non-linear case

gradf=lgradFreg(data.X,data.Y,vars0.w,vars0.w0,F)+gradOmega2(F,data.regnorm2)+data.regnormdiff*gradOmegadiff(F);


end
