function [x,cost,vars,opts]=grad_desc_update(x0,vars0,data,cost0,opts)
% [x,cost,vars]=grad_desc_update(x0,vars0,cost0,opts)
% function doing a line_search (golden) using the cost function and
% gradient function defined in opts.
% input:
%   x0 : value of x before the descent (x may be up to dim 3)
%   vars0: stucture needed by the cost function and the gradient function
%          (this structure can be updated for warm start or not )
%   data : constant data used in the cost and gradient function
%   cost0: cost at point x0
%   opts : options fior the gradient descent
%   opts.costfunc: function for the cost that is used like that :
%           [costnew,varsnew]=costfunc(xnew,data,vars);
%   opts.gradfunc: function returning the gradient:
%           [gradf]=gradfunc(x0,data,vars0)
%   opts.use_cg : contains wether or not to use conjugate gradient
%                 if not zero then the number is the type of cg (see code)
%   opts.goldensearch_stepmax: the max step of the golden search
%   opts.goldensearch_deltmax: the max delta of the golden search
%   opts.g_1: last gradient direction (for cg else = 1)
%   opts.d_1: last descent direction (for cg else = 0)
%
%  Author : Remi Flamary remi (dot) flamary (at) gmail (dot) com

if ~isfield(opts,'projfunc'); opts.projfunc=@proj_0; end
if ~isfield(opts,'linesearch'); opts.linesearch=0; end
if ~isfield(opts,'testgrad'); opts.testgrad=0; end

%x=x0;

switch opts.linesearch

    case 1 % Armijo... opts.ls_tol, opts.ls_stepmin
        line_search=@line_search_armijo;

    case 2 % Dennis... opts.ls_tol, opts.ls_stepmin
        line_search=@line_search_dennis;

    case 3 %
        line_search=@line_search_golden2;

    otherwise % golden search opts: opts.goldensearch_stepmax,opts.goldensearch_deltmax
        line_search=@line_search_golden2;
end


%varsdeb=vars0;

CostOld=cost0;


g=opts.gradfunc(x0,data,vars0);

g_1=opts.g_1;
d_1=opts.d_1;

% conjugate gradient ...
switch opts.use_cg

    case 1 % Fletcher Reeves 1964
        b=sum(sum(sum(g.^2)))/sum(sum(sum(g_1.^2)));

    case 2 % Ploak-Ribière-Polyak 1969
        y=g-g_1;
        b=sum(sum(sum(g.*y)))/sum(sum(sum(g_1.^2)));

    case 3 % Hestenes-Stiefel 1959
        y=g-g_1;
        b=sum(sum(sum(g.*y)))/sum(sum(sum(d_1.*y)));

    case 4 % Fletcher 1987 (conjugate dscent)
        b=-sum(sum(sum(g.^2)))/sum(sum(sum(g_1.*d_1)));

    case 5 % Liu and Storey 1991
        y=g-g_1;
        b=-sum(sum(sum(g.*y)))/sum(sum(sum(g_1.*d_1)));

    case 6 % Day Yuan 1999
        y=g-g_1;
        b=sum(sum(sum(g.^2)))/sum(sum(sum(d_1.*y)));

    case 7 % Hager Zhang 2005
        y=g-g_1;
        b=sum(sum(sum((y-2*d_1*sum(sum(sum(y.^2))))/sum(sum(sum(d_1.*y))).*(g/sum(sum(sum(d_1.*y)))))));

    otherwise
        b=0;
end

if (g_1(1,1)==1) && (d_1(1,1)==0)
    b=0;
end


DF=-g+b*d_1;

d=DF;

normgrad=sum(sum(sum(DF.*DF)));

if normgrad~=0
    DF=DF/sqrt(normgrad);
end

desc=DF;

vars0.dg=sum(sum(sum(desc.*g)));

if opts.testgrad~=0
   temp=opts.costfunc(opts.projfunc(x0,desc,-opts.testgrad,data,vars0),data,vars0);
   if temp<cost0
       fprintf('Wrong gradient descent direction!\n');
   end
end


% line search
[cost,step,vars]=line_search(x0,desc,CostOld,data,vars0,opts);

x=opts.projfunc(x0,desc,step,data,vars);

opts.g_1=g;
opts.d_1=d;
%opts.goldensearch_deltmax
end


function [x]=proj_0(x0,desc,step,data,vars)
x=step*desc+x0;
end


function [cost,step,vars]=line_search_golden(x0,desc,CostOld,data,vars0,opts)


gold = (sqrt(5)+1)/2 ;

stepmax = opts.goldensearch_stepmax;
deltmax = stepmax;

%----------------------------------------------------
% Compute optimal stepsize
%-----------------------------------------------------
stepmin  = 0;
costmin  = CostOld ;
[costmax,varsmax] = opts.costfunc(opts.projfunc(x0,desc,stepmax,data,vars0),data,vars0);

%-----------------------------------------------------
% maximum stepsize
%-----------------------------------------------------


%-----------------------------------------------------
%  Linesearch
%-----------------------------------------------------

Step = [stepmin stepmax];
Cost = [costmin costmax];
Varsall = [vars0 varsmax]; varsmin=vars0;

[val,coord] = min(Cost);
% optimization of stepsize by golden search
while (stepmax-stepmin)>opts.goldensearch_deltmax*(abs(deltmax))  & stepmax > eps;
    stepmedr = stepmin+(stepmax-stepmin)/gold;
    stepmedl = stepmin+(stepmedr-stepmin)/gold;

    [costmedr,varsr] = opts.costfunc(opts.projfunc(x0,desc,stepmedr,data,varsmin),data,varsmin) ;
    [costmedl,varsl] = opts.costfunc(opts.projfunc(x0,desc,stepmedl,data,varsmax),data,varsmax) ;

    %costmedr=-costmedr;
    %costmedl=-costmedl;
    Step = [stepmin stepmedl stepmedr stepmax];
    Cost = [costmin costmedl costmedr costmax];
    Varsall=[varsmin varsl varsr varsmax];

    [val,coord] = min(Cost);
    switch coord
        case 1
            stepmax = stepmedl;
            costmax = costmedl;
            varsmax = varsl;
        case 2
            stepmax = stepmedr;
            costmax = costmedr;
            varsmax = varsr;
        case 3
            stepmin = stepmedl;
            costmin = costmedl;
            varsmin = varsl;
        case 4
            stepmin = stepmedr;
            costmin = costmedr;
            varsmin = varsr;
    end;
end;


%---------------------------------
% Final Updates
%---------------------------------

cost = Cost(coord) ;
step = Step(coord) ;
vars = Varsall(coord);


end


function [cost,step,vars]=line_search_armijo(x0,desc,CostOld,data,vars0,opts)

% TODO

if ~isfield(opts,'ls_tol'); opts.ls_tol=1.e-4; end
if ~isfield(opts,'ls_stepmin'); opts.ls_stepmin=1.e-8; end

stpmin=opts.ls_stepmin;
ftol=opts.ls_tol;

iprint  = 0;
fid     = 1;

%initialize
iter   = 0;
f0     = CostOld;
%x0     = x0;

stp=2;

loop=1;

vars=vars0;

while (stp > stpmin)&&loop

    % reduce step size
    stp  = 0.5*stp;
    iter = iter + 1;

    %  trial iterate
    %x = x0 + stp * d;
    x=opts.projfunc(x0,desc,stp,data,vars);

    %  compute the function value at x+stp*d
    [f,vars] = opts.costfunc(x,data,vars) ;

    %  print iteration info
    % if( iprint > 0 )
    %     fprintf(fid, '%4d    %12.5e    %12.5e   %12.5e \n', ...
    %                   iter, stp, f, f0 + stp*ftol*dg )
    % end

    %  check decrease condition and return if successful
    if( f < f0 + stp*ftol*vars0.dg )
        loop=0;
    end


end


if f<f0

    cost=f;
    step=stp;

else

    vars=vars0;
    cost=CostOld;
    step=0;

end

end

function [cost,step,vars]=line_search_dennis(x0,desc,CostOld,data,vars0,opts)

% TODO

if ~isfield(opts,'ls_tol'); opts.ls_tol=1.e-4; end
if ~isfield(opts,'ls_stepmin'); opts.ls_stepmin=1.e-8; end

stpmin=opts.ls_stepmin;
ftol=opts.ls_tol;

iprint  = 0;
fid     = 1;

stp=1;

%initialize
iter   = 0;
f0     = CostOld;
%x0     = x0;
stp_c  = stp;
fc     = CostOld;

dg=vars0.dg;

loop=1;

vars=vars0;

while (stp > stpmin)&&loop

    %  trial iterate
    x=opts.projfunc(x0,desc,stp,data,vars);

    %  compute the function value at x+stp*d
    [f,vars] = opts.costfunc(x,data,vars) ;


    %  check decrease condition and return if successful
    if( f < f0 + stp*ftol*dg )
        loop=0;
    end

    if loop==1
        stp_p  = stp_c;
        stp_c  = stp;
        fp     = fc;
        fc     = f;

        %  compute next step size stp
        if( iter == 0 )
            %  first backtrack: quadratic fit
            stp = - dg / ( 2 * (fc - f0 - dg) );
        else
            %  all subsequent backtracks: cubic fit

            t1 = fc - f0 - stp_c*dg;
            t2 = fp - f0 - stp_p*dg;
            t3 = 1 / (stp_c - stp_p);

            a = t3 * ( t1/(stp_c^2) - t2/(stp_p^2) );
            b = t3 * ( t2*stp_c/(stp_p^2) - t1*stp_p/(stp_c^2) );
            disc = b^2 - 3*a*dg;

            if( a ~= 0 )
                %  cubic has unique minimum
                stp = (-b + sqrt( disc ) ) / (3 * a );
            else
                %  cubic is a quadratic
                stp = -dg / ( 2 * b );
            end

        end

        %  saveguard the step size
        if( stp > 0.5*stp_c ); stp = 0.5*stp_c; end
        if( stp < stp_c/10 );  stp = stp_c/10; end

        iter = iter + 1;
    end
end


if f<f0

    cost=f;
    step=stp;

else

    vars=vars0;
    cost=CostOld;
    step=0;

end

end

function [cost,step,vars]=line_search_golden2(x0,desc,CostOld,data,vars0,opts)


gold = (sqrt(5)+1)/2 ;

stepmax = opts.goldensearch_stepmax;
deltmax = stepmax;

%----------------------------------------------------
% Compute optimal stepsize
%-----------------------------------------------------
stepmin  = 0;
costmin  = CostOld ;
[costmax,varsmax] = opts.costfunc(opts.projfunc(x0,desc,stepmax,data,vars0),data,vars0);

%-----------------------------------------------------
% maximum stepsize
%-----------------------------------------------------


%-----------------------------------------------------
%  Linesearch
%-----------------------------------------------------

%initialisation

varsmin=vars0;

stepmedr = stepmin+(stepmax-stepmin)/gold;
stepmedl = stepmin+(stepmedr-stepmin)/gold;

[costmedr,varsr] = opts.costfunc(opts.projfunc(x0,desc,stepmedr),data,vars0) ;
[costmedl,varsl] = opts.costfunc(opts.projfunc(x0,desc,stepmedl),data,vars0) ;

Step = [stepmin stepmedl stepmedr stepmax];
Cost = [costmin costmedl costmedr costmax];
Varsall=[varsmin varsl varsr varsmax];

[val,coord] = min(Cost);

loop=1;

% optimization of stepsize by golden search
while loop


    if (stepmax-stepmin)>opts.goldensearch_deltmax*(abs(deltmax))  && stepmax > eps;
        loop=1;
    else
        loop=0;
    end

    [val,coord] = min(Cost);

    if loop

        switch coord
            case 1
                % new steps
                stepmax = stepmedl;

                stepmedr = stepmin+(stepmax-stepmin)/gold;
                stepmedl = stepmin+(stepmedr-stepmin)/gold;

                % on garde les couts
                costmax = costmedl;
                varsmax = varsl;

                [costmedr,varsr] = opts.costfunc(opts.projfunc(x0,desc,stepmedr,data,varsmax),data,varsmax) ;
                [costmedl,varsl] = opts.costfunc(opts.projfunc(x0,desc,stepmedl,data,varsmin),data,varsmin) ;


            case 2
                stepmax = stepmedr;
                costmax = costmedr;
                costmedr=costmedl;
                varsmax = varsr;
                varsr=varsl;

                stepmedr = stepmin+(stepmax-stepmin)/gold;
                stepmedl = stepmin+(stepmedr-stepmin)/gold;

                [costmedl,varsl] = opts.costfunc(opts.projfunc(x0,desc,stepmedl,data,varsr),data,varsr) ;

            case 3
                stepmin = stepmedl;
                costmin = costmedl;
                costmedl=costmedr;
                varsl=varsr;
                varsmin = varsl;

                stepmedr = stepmin+(stepmax-stepmin)/gold;
                stepmedl = stepmin+(stepmedr-stepmin)/gold;

                [costmedr,varsr] = opts.costfunc(opts.projfunc(x0,desc,stepmedr,data,varsl),data,varsl) ;

            case 4
                stepmin = stepmedr;
                costmin = costmedr;
                varsmin = varsr;

                stepmedr = stepmin+(stepmax-stepmin)/gold;
                stepmedl = stepmin+(stepmedr-stepmin)/gold;

                [costmedr,varsr] = opts.costfunc(opts.projfunc(x0,desc,stepmedr,data,varsmax),data,varsmax) ;
                [costmedl,varsl] = opts.costfunc(opts.projfunc(x0,desc,stepmedl,data,varsmin),data,varsmin) ;

        end;

        Step = [stepmin stepmedl stepmedr stepmax];
        Cost = [costmin costmedl costmedr costmax];
        Varsall=[varsmin varsl varsr varsmax];

    end
end;


%---------------------------------
% Final Updates
%---------------------------------

cost = Cost(coord) ;
step = Step(coord) ;
vars = Varsall(coord);


end
