function [gmkl,obj]=gmklclass(X,Y,options)
% Generalized Multiple Kernel Learning wrapper function.
%
% [gmkl,obj]=gmklclass(X,Y,options)
%
%  options.kerneltype :  0 sum of gaussian kernel (default)
%                        1 product of gaussian kernels
%                        2 sum of precomputed kernels* (uses global
%                        Kprecomp)
%                        3 product of exp of precomputed kernels
%
%  options.regtype :     0 l1 regularization (default)
%                        1 l2 regularization
%
%  options.prob :        0 : classification SVC
%                        3 : regression eps-SVR
%
%  options.C:            C regularization parameter
%  optiosn.verbose :     verbose level
% options.nbitermax : max number of iteration

if ~isfield(options,'C'); options.C=10; end
if ~isfield(options,'kerneltype'); options.kerneltype=0; end
if ~isfield(options,'regtype'); options.regtype=0; end
if ~isfield(options,'prob'); options.prob=0; end
if ~isfield(options,'verbose'); options.verbose=0; end

if Y(1)>0
    sigclass=1;
else
    sigclass=-1;
end

Y=sigclass*Y;

  NUMkernels=size(X,2);
  parms=initparms(NUMkernels,options.kerneltype,options.regtype);

  parms.C=options.C/length(Y);
  parms.BOOLverbose=(options.verbose~=0);
  %parms.

  svm=COMPGDoptimize(X',Y',parms);
  
  gmkl=svm;
  gmkl.xsup=X(svm.svind,:);
  gmkl.w=sigclass*svm.alphay;
  gmkl.pos=svm.svind;
  gmkl.kernel='gaussian';
  gmkl.kerneloption=1;
  gmkl.w0=sigclass*svm.b;
  gmkl.gamma=svm.d;
  gmkl.kerneltype=options.kerneltype;
  gmkl.regtype=options.regtype;
  
  gmkl.alphay=sigclass*gmkl.alphay;
  gmkl.b=sigclass*gmkl.b;
  obj=svm.totalcost;
  
  
  
end

function D=precomp(TRNfeatures,TSTfeatures)
  [NUMk,NUMtrain]=size(TRNfeatures);
  [NUMk,NUMtest]=size(TSTfeatures);
  D=zeros(NUMtrain,NUMtest,NUMk);
  for k=1:NUMk, D(:,:,k)=(repmat(TRNfeatures(k,:)',1,NUMtest)-repmat(TSTfeatures(k,:),NUMtrain,1)).^2; end;
end

function parms=initparms(NUMk,TYPEker,TYPEreg)
  % Kernel parameters
  switch (TYPEker),
    case 0, parms.KERname='Sum of RBF kernels across features';
            parms.gamma=ones(NUMk,1);
	    parms.fncK=@KSumRBF;
	    parms.fncKdash=@KdashSumRBF;
	    parms.fncKdashint=@KdashSumRBFintermediate;
    case 1, parms.KERname='Product of RBF kernels across features';
            parms.gamma=1;
	    parms.fncK=@KProdRBF;
	    parms.fncKdash=@KdashProdRBF;
	    parms.fncKdashint=@KdashProdRBFintermediate;
    case 2, parms.KERname='Sum of precomputed kernels';
	    parms.fncK=@KSumPrecomp;
	    parms.fncKdash=@KdashSumPrecomp;
	    parms.fncKdashint=@KdashSumPrecompintermediate;
    case 3, parms.KERname='Product of exponential kernels of precomputed disance matrices';
            parms.gamma=1;
	    parms.fncK=@KProdExpPrecomp;
	    parms.fncKdash=@KdashProdExpPrecomp;
	    parms.fncKdashint=@KdashProdExpPrecompintermediate;
    otherwise, fprintf('Unknown kernel type.\n'); keyboard; 
  end;
      
  % Regularization parameters
  switch (TYPEreg),
    case 0, parms.REGname='l1';          % L1 Regularization
            parms.sigma=ones(NUMk,1);
	    parms.fncR=@Rl1;
	    parms.fncRdash=@Rdashl1;
    case 1, parms.REGname='l2';          % L2 Regularization
            parms.mud=ones(NUMk,1);
	    parms.covd=1e-1*eye(NUMk);
	    parms.invcovd=inv(parms.covd);
	    parms.fncR=@Rl2;
	    parms.fncRdash=@Rdashl2;
    otherwise, fprintf('Unknown regularisation type.\n'); keyboard; 
  end;
    
  % Standard SVM parameters
  parms.TYPEprob=0;           % 0 = C-SVC, 3 = EPS-SVR (try others at your own risk)
  parms.C=10;                 % Misclassification penalty for SVC/SVR ('-c' in libSVM)
  
  % Gradient descent parameters
  parms.initd=rand(NUMk,1);   % Starting point for gradient descent
  parms.TYPEsolver='LIB';     % Use Matlab's quadprog as an SVM solver.
  parms.TYPEstep=1;           % 0 = Armijo, 1 = Variant Armijo, 2 = Hessian (not yet implemented)
  parms.MAXITER=40;           % Maximum number of gradient descent iterations
  parms.MAXEVAL=1000;          % Maximum number of SVM evaluations
  parms.MAXSUBEVAL=20;        % Maximum number of SVM evaluations in any line search
  parms.SIGMATOL=0.3;         % Needed by Armijo, variant Armijo for line search
  parms.BETAUP=2.1;           % Needed by Armijo, variant Armijo for line search
  parms.BETADN=0.3;           % Needed by variant Armijo for line search
  parms.BOOLverbose=true;     % Print debug information at each iteration
  parms.SQRBETAUP=parms.BETAUP*parms.BETAUP;    
end

function r=Rl1(parms,d), r=parms.sigma'*d; end
function rdash=Rdashl1(parms,d), rdash=parms.sigma; end

function r=Rl2(parms,d), delta=parms.mud-d;r=0.5*delta'*parms.invcovd*delta; end
function rdash=Rdashl2(parms,d), rdash=parms.invcovd*(d-parms.mud); end

function K=KSumRBF(TRNfeatures,TSTfeatures,parms,d)
  [NUMdims,NUMtrn]=size(TRNfeatures);
  [NUMdims,NUMtst]=size(TSTfeatures);
  if (NUMdims~=length(d)), fprintf('NUMdims not equal to NUMk\n'); keyboard; end;
  
  nzind=find(d>1e-4);
  K=zeros(NUMtrn,NUMtst);
  if (~isempty(nzind)),
    for i=1:length(nzind),
      k=nzind(i);
      Dk=(repmat(TRNfeatures(k,:)',1,NUMtst)-repmat(TSTfeatures(k,:),NUMtrn,1)).^2;
      K=K+d(k)*exp(-parms.gamma(k)*Dk);
    end;
  end;
end

function Kdashint=KdashSumRBFintermediate(TRNfeatures,TSTfeatures,parms,d)
  Kdashint=[];
end

function Kdash=KdashSumRBF(TRNfeatures,TSTfeatures,parms,d,k,Kdashint)
  [NUMdims,NUMtrn]=size(TRNfeatures);
  [NUMdims,NUMtst]=size(TSTfeatures);

  Kdash=exp(-parms.gamma(k)*(repmat(TRNfeatures(k,:)',1,NUMtst)-repmat(TSTfeatures(k,:),NUMtrn,1)).^2);
end

function K=KProdRBF(TRNfeatures,TSTfeatures,parms,d)
  [NUMdims,NUMtrn]=size(TRNfeatures);
  [NUMdims,NUMtst]=size(TSTfeatures);
  if (NUMdims~=length(d)), fprintf('NUMdims not equal to NUMk\n'); keyboard; end;
  
  nzind=find(d>1e-4);
  K=zeros(NUMtrn,NUMtst);
  if (~isempty(nzind)),
    for i=1:length(nzind),
      k=nzind(i);
      K=K+d(k)*(repmat(TRNfeatures(k,:)',1,NUMtst)-repmat(TSTfeatures(k,:),NUMtrn,1)).^2;
    end;
  end;
  K=exp(-parms.gamma*K);
end

function Kdashint=KdashProdRBFintermediate(TRNfeatures,TSTfeatures,parms,d)
  Kdashint=-parms.gamma*parms.fncK(TRNfeatures,TSTfeatures,parms,d);
end

function Kdash=KdashProdRBF(TRNfeatures,TSTfeatures,parms,d,k,Kdashint)
  [NUMdims,NUMtrn]=size(TRNfeatures);
  [NUMdims,NUMtst]=size(TSTfeatures);

  Kdash=Kdashint.*(repmat(TRNfeatures(k,:)',1,NUMtst)-repmat(TSTfeatures(k,:),NUMtrn,1)).^2;
end

function K=KSumPrecomp(TRNfeatures,TSTfeatures,parms,d)
  global Kprecomp;
  if (size(Kprecomp,3)~=length(d)), fprintf('Number of kernels does not equal number of weights.\n');keyboard; end;

  K=lincomb(Kprecomp(TRNfeatures,TSTfeatures,:),d);
end

function Kdashint=KdashSumPrecompintermediate(TRNfeatures,TSTfeatures,parms,d)
  Kdashint=[];
end

function Kdash=KdashSumPrecomp(TRNfeatures,TSTfeatures,parms,d,k,Kdashint)
  global Kprecomp;
  
  Kdash=Kprecomp(TRNfeatures,TSTfeatures,k);
end

function K=KProdExpPrecomp(TRNfeatures,TSTfeatures,parms,d)
  global Dprecomp;
  if (size(Dprecomp,3)~=length(d)), fprintf('Number of kernels does not equal number of weights.\n');keyboard; end;

  K=exp(-parms.gamma*lincomb(Dprecomp(TRNfeatures,TSTfeatures,:),d));
end

function Kdashint=KdashProdExpPrecompintermediate(TRNfeatures,TSTfeatures,parms,d)
  Kdashint=-parms.gamma*parms.fncK(TRNfeatures,TSTfeatures,parms,d);
end

function Kdash=KdashProdExpPrecomp(TRNfeatures,TSTfeatures,parms,d,k,Kdashint)
  global Dprecomp;
  
  Kdash=Kdashint.*Dprecomp(TRNfeatures,TSTfeatures,k);
end

function K=lincomb(base,d)
  nzind=find(d>1e-4);
  K=zeros(size(base,1),size(base,2));
  if (~isempty(nzind)), for k=1:length(nzind), K=K+d(nzind(k))*base(:,:,nzind(k)); end; end;
end
