function [filtersvm,LOG]=adaptivescaling_svmclass(x,y,options)
% function thta permits to learn an optimal scaling of features inside
% gaussian kernel
% classification
% Parameters:
% x : data
% y : classes( -1,1)
% options:
%  options.gamma : initialisation ...
%  options.kernel: kernel : gaussian of course
%  options.kerneloption: kernel option
%  options.lambda : lambda value for svm
%  options.C : C value for svm
%  options.verbose: print everything
%  options.log : log informations
%  options.nbitermax : log informations
%  options.stopcF : threshold for stopping on F changes
%  options.decy : y time shift...
%  options.solver : solver choice for non-linear svm
%
% Result:
% sigsvm:
%  sigsvm.kernel : kernel type in SVM-KM format
%                   'gaussian' or 'linear' leading to primal or dual
%                    optimization
%  sigsvm.xsup : support vector
%  sigsvm.w    : dual or primal variables weight
%  sigsvm.w0   : dual or primal zero value
%  sigsvm.pos  : position of support vectors (if non linear)
%  sigsvm.decy : time shift
%
%  Author : Remi Flamary remi (dot) flamary (at) gmail (dot) com



if ~isfield(options,'gamma'); options.gamma=ones(1,size(x,2)); end

if ~isfield(options,'kernel'); options.kernel='linear'; end
if ~isfield(options,'kerneloption'); options.kerneloption=1; end
if ~isfield(options,'lambda'); options.lambda=1e-10; end
if ~isfield(options,'C'); options.C=100; end
if ~isfield(options,'verbose'); options.verbose=0; end
if ~isfield(options,'log'); options.log=0; end
if ~isfield(options,'nbitermax'); options.nbitermax=50; end
if ~isfield(options,'decy'); options.decy=0; end
if ~isfield(options,'alphainit'); options.alphainit=[]; end
if ~isfield(options,'solver'); options.solver=0; end
if ~isfield(options,'numericalprecision'); options.numericalprecision=1e-15; end
if ~isfield(options,'use_cg'); options.use_cg=0; end
if ~isfield(options,'regnorm'); options.regnorm=1; end
if ~isfield(options,'pnorm'); options.pnorm=1; end
if ~isfield(options,'use_proj'); options.use_proj=0; end

if ~isfield(options,'multiclass'); options.multiclass=0; end

if length(unique(y))>2
    options.multiclass=1;
end

if length(options.gamma)==1
	options.gamma=options.gamma*ones(1,size(x,2))/size(x,2);
end

t0=cputime;

if options.multiclass==0
    % initialization

    LOG=[];
    
    gamma=options.gamma;

    %Xt=mfilter(F,X);

    % counter initialisation
    global nbcall
    nbcall=0;

    % data initialisation
    data=options;
    data.X=x;
    data.Y=y;
    data.options=options;


    % solver options initialisation
    opts=options;

        opts.costfunc=@costfunc;
        opts.gradfunc=@gradfunc;
        opts.printfunc=@printfunc;
        
        if opts.use_proj~=0
            opts.projfunc=@proj_sup0;
        end
        
        


    % solving

        % solving standard l2 problem
        [gamma,vars,costmin,LOG2]=grad_desc(gamma,[],data,opts);



    if options.log==1

        LOG.Ftot=LOG2.xtot;
        LOG.gamma=options.gamma;
        LOG.decy=options.decy;
        LOG.J=LOG2.J;
        LOG.time=LOG2.time;
        LOG.xsup=LOG2.vars.xsup;
        LOG.pos=LOG2.vars.pos;
        LOG.w=LOG2.vars.w;
        LOG.w0=LOG2.vars.w0;
        LOG.kernel=options.kernel;
        LOG.kerneloption=options.kerneloption;
        LOG.nbcostcall=nbcall;
    end

    % returned structure
    filtersvm.kernel=options.kernel;
    filtersvm.kerneloption=options.kerneloption;
    filtersvm.xsup =vars.xsup;
    filtersvm.w   =vars.w;
    filtersvm.w0  =vars.w0;
    filtersvm.pos =vars.pos;
    filtersvm.gamma   =gamma;



else % multiclass

    vals=sort(unique(y),'ascend');

    if options.verbose~=0
        disp(['Multiclass SVM classification (' num2str(length(vals)) 'classes)' ]);
    end

    switch options.multiclass

        case 1 % one against all


            options2=options;
            options2.multiclass=0;
            
            for i=1:length(vals)

                if options.verbose~=0
                    disp(['Classification :' num2str(i) '/' num2str(length(vals)) ]);
                end;
                Yt=(y==vals(i))-(y~=vals(i));
                [models{i},LOG{i}]=adaptivescaling_svmclass(x,Yt,options2);

            end

            filtersvm.multiclass=options.multiclass;
            filtersvm.nbclass=length(vals);
            filtersvm.models=models;
            filtersvm.kerneloption=options.kerneloption;
            filtersvm.kernel=options.kernel;

    end




end

end


function [costnew,varsnew]=costfunc(gamma,data,vars)
% cost function for the non-linear case

%cost counter
global nbcall
nbcall=nbcall+1;

varsnew=vars;

% filtering data
Xf=data.X*diag(gamma);

% initialize alphas
alphainit=zeros(size(data.Y));
if ~isempty(vars)
    alphainit(vars.pos)=abs(vars.w);
end
%data.alphainit=alphainit;
options=SetFields([],data,{'kernel','kerneloption','C','lambda','solver'});
options.verbose=0;
options.alphainit=alphainit;


% solve with warm start
[svm,obj]=svmclass2(Xf,data.Y,options);


%extract suport vectors
%xtemp=xsup;
%Ktemp=svmkernel(xtemp,data.kernel,data.kerneloption);

% cost calculation
costnew=obj+data.regnorm*Omegap(gamma,data.pnorm);

varsnew.pos=svm.pos;
varsnew.w=svm.w;
varsnew.w0=svm.w0;
varsnew.xsup=svm.xsup;

end

function [gradf]=gradfunc(gamma,data,vars0)
% gradient function for the non-linear case

alpha=zeros(length(data.Y),1);
alpha(vars0.pos)=abs(vars0.w);

gradf=grad_gamma_gaussian(data.X,alpha.*data.Y,gamma,data)+data.regnorm*gradOmegap(gamma,data.pnorm);


end

function [nbverbose]=printfunc(nbverbose,verbose,J,deltaJ,vars)

if isempty(vars)
    vars.w=[];
end

nbverbose = nbverbose+1;
if nbverbose == 20 || (nbverbose==1 && deltaJ==0);
    disp([repmat('    ',1,verbose-1) '      Cost     Delta Cost  nbsup']);
    nbverbose = 0;
end


fprintf(1,[repmat('    ',1,verbose-1) '| %11.4e | %8.4f | %d |\n'],[J deltaJ length(vars.w)]);

end

function [x]=proj_sup0(x0,desc,step)
x=max(step*desc+x0,0);
end





